@@ -2,7 +2,8 @@ from eventlet import patcher
 from eventlet.green import subprocess
 from eventlet.green import time
 
-patcher.inject('test.test_subprocess',
+patcher.inject(
+    'test.test_subprocess',
     globals(),
     ('subprocess', subprocess),
     ('time', time))
